@@ -690,12 +690,27 @@ class Sf:
     
     async def field_permission_user_bulk_check(self, objects: list, permissions: list,  users= [], ):
         data = {}
-        async_field_permission_check = async_wrap(self.field_permission_user_check)
-        result_list = await asyncio.gather(*[async_field_permission_check(object, permission,  users ) for object in objects for permission in permissions])
-        # for result in result_list:
-        #     data = deep_merge_dictionaries(data, result)
-        result_list = [item for sublist in result_list for item in sublist]
-        return result_list
+        task_dict = {}
+        for object in objects:
+            for permission in permissions:
+                task_dict[f"{object};;{permission}"] = asyncio.create_task(self.field_permission_user_check(object, permission, users))
+        
+        gathered_tasks = await asyncio.gather(*task_dict.values())
+        
+        task_results_dict = dict(zip(task_dict.keys(), gathered_tasks))
+        
+        
+        result = []
+        
+        for key, value in task_results_dict.items():
+            object, permission = key.split(';;')
+            value['Object API Name'] = object
+            value['Permission'] = permission
+            result.append(value)
+
+        result_pd = pd.concat(result, ignore_index=True)
+        result_pd.columns = ['UserName','Profile','Field API Name','Permission Set','Object API Name','Permission']
+        return result_pd
 
     async def entity_access_bulk_query(self, entity_types: list):
         async_entity_access_query = async_wrap(self.entity_access_query)
@@ -1127,7 +1142,7 @@ class Sf:
         
         return {'permissionSetPermissions': permissionSetPermissions, 'profilePermissions': profilePermissions}
 
-    def field_permission_user_check(self, object, permission, users=[]):
+    async def field_permission_user_check(self, object, permission, users=[]):
         # Similar to the field_permission_check function, but this one will return a list of users that have the specified permission on the specified object
         
         relevant_ps_and_profiles = transform_sf_result_set_rec(self.sf.query_all(f"""
@@ -1155,35 +1170,54 @@ class Sf:
                                                                 WHERE PermissionSetId
                                                                 IN ('{relevant_ps_and_profiles_list}')
                                                                 {user_query} AND Assignee.IsActive = TRUE""")['records'])
+        
+        # Retrieve Profile FLS data
+        profiles = data[data['PermissionSet.IsOwnedByProfile']]['Profile.Name'].unique().tolist()
+        profiles = ['Admin' if x == 'System Administrator' else x for x in profiles]
 
+
+        async_metadata_read = async_wrap(sf.metadata_read)
+        profile_metadata_tasks = {}
+        profile_metadata_dict = {}
+
+        for profile in profiles:
+                profile_metadata_tasks[profile] = asyncio.create_task(async_metadata_read('Profile', profile))
+
+
+        task_result_list = await asyncio.gather(*profile_metadata_tasks.values())
+
+
+        for profile in profiles:
+                profile_metadata_dict["System Administrator" if profile == 'Admin' else profile] = pd.DataFrame(task_result_list[profiles.index(profile)]['fieldPermissions'][0])
+    
         data = data.merge(relevant_ps_and_profiles, left_on='PermissionSet.Id', right_on='FieldPermissions.ParentId', how='left').copy()
 
         if (len(data) > 0):
-                profile_violations = data[data['PermissionSet.IsOwnedByProfile']== True][['User.Username', 'Profile.Name','Field API Name']]
+                
+                profile_violations = data[data['PermissionSet.IsOwnedByProfile']== True][['User.Username', 'Profile.Name']].drop_duplicates().copy()
+                # profile_violations = data[data['PermissionSet.IsOwnedByProfile']== True][['User.Username', 'Profile.Name','Field API Name']]
+                profile_metadata_pd = pd.concat(profile_metadata_dict.values(), keys=profile_metadata_dict.keys(), names=['Profile', 'Index']).reset_index(level=1, drop=True).reset_index()
+                profile_metadata_pd.rename(columns={'Profile': 'Profile.Name'}, inplace=True)
+                profile_metadata_pd['Field API Name'] = profile_metadata_pd['field'].str.split('.').str[1]
+                profile_metadata_pd['Object API Name'] = profile_metadata_pd['field'].str.split('.').str[0]
+                profile_metadata_pd = profile_metadata_pd[profile_metadata_pd['Object API Name'] == object].copy()
+                profile_metadata_pd.drop(columns=['field', 'Object API Name'], inplace=True)
+
+
+                if permission == 'PermissionsEdit':
+                        profile_metadata_pd = profile_metadata_pd[profile_metadata_pd['editable'] == True].copy()
+                        profile_metadata_pd.drop(columns=['editable', 'readable'], inplace=True)
+                elif permission == 'PermissionsRead':
+                        profile_metadata_pd = profile_metadata_pd[profile_metadata_pd['readable'] == True].copy()
+                        profile_metadata_pd.drop(columns=['editable', 'readable'], inplace=True)
+                
+                profile_violations = profile_violations.merge(profile_metadata_pd, on='Profile.Name', how='left').copy()
                 permission_set_violations = data[data['PermissionSet.IsOwnedByProfile']== False][['User.Username', 'PermissionSet.Label', 'Field API Name']]
-                result = []
-                for field in profile_violations['Field API Name'].unique():
-                        for user in profile_violations['User.Username'].unique():
-                                result.append({"UserName": user, 
-                                                "Object API Name" : object,
-                                                "Field API Name" : field,
-                                                "Permission": permission,
-                                                "Profile": "; ".join(list(profile_violations[profile_violations['User.Username'] == user]['Profile.Name'].unique())),
-                                                "Permission Set": ""
-                                                
-                                                }
-                                                
-                                                )
-                        for user in permission_set_violations['User.Username'].unique():
-                                result.append({"UserName": user, 
-                                                "Object API Name" : object,
-                                                "Field API Name" : field,
-                                                "Permission": permission,
-                                                "Profile": "",
-                                                "Permission Set": "; ".join(list(permission_set_violations[permission_set_violations['User.Username'] == user]['PermissionSet.Label'].unique()))})
-                return result
+        
+                return pd.merge(profile_violations, permission_set_violations, on=['User.Username','Field API Name'], how='outer')
         else:
-                return []
+                return pd.DataFrame(columns=['User.Username', 'Profile.Name', 'Field API Name', 'PermissionSet.Label'])
+            
             
     # Retrieves the Page Layout by Profile by Record Type data for all objects in the org
     def retrieve_profile_layout_record_type_matrix_all(self, return_pivot_table = False):
