@@ -227,11 +227,15 @@ class Vn:
         self.networkPassword = self.networkPassword if networkPassword is None else networkPassword
         self.networkCountry = self.networkCountry if networkCountry is None else networkCountry
         
-        if (self.networkUserName and self.networkPassword):
+        if (self.networkUserName and self.networkPassword and self.networkURL):
             self.networkConnection = requests.post(self.networkURL + '/api/v17.0/auth?username=' + self.networkUserName + '&password=' + self.networkPassword)
-            self.sessionId = self.networkConnection.json()['sessionId']
-            self.userId = self.networkConnection.json()['userId']
-            self.networkId = self.networkConnection.json()['networkId']
+            if (self.networkConnection.json()['responseStatus'] == 'SUCCESS'):
+                self.sessionId = self.networkConnection.json()['sessionId']
+                self.userId = self.networkConnection.json()['userId']
+                self.networkId = self.networkConnection.json()['networkId']
+            else:
+                raise Exception(self.networkConnection.json())
+
         
         self.APIheaders = {'authorization': self.sessionId}
         self.APIversionList = []
@@ -295,9 +299,9 @@ class Vn:
         self,
         referenceTypes: Optional[list] = None, # List of reference types to retrieve, leave empty (None) to retrieve all
         customerOwned: Optional[bool] = None, # True (customer owned only), False (ootb), None (customer owned and ootb)
-        activeReferences: Optional[bool ]= None, # True (active only), False (inactive only), None (active and inactive)
+        activeReferences: Optional[bool]= None, # True (active only), False (inactive only), None (active and inactive)
         languages: Optional[list]= None, # List of languages to retrieve, leave empty (None) to retrieve all
-        countries: Optional[list ]= None, # List of countries to retrieve, leave empty (None) to retrieve all
+        countries: Optional[list]= None, # List of countries to retrieve, leave empty (None) to retrieve all
     ):
         """
         Processes Network Reference Metadata and returns a DataFrame of the reference codes based on the parameters.
